# KMS

This is a prototypically Key Management System (KMS) for Quantum Key Distribution Networks (QKDN).

It uses, by now, an emulation of a quantum link to exchange random numbers between neighboring, i.e., directly attached, quantum network elements.

**Note well** _This is not intended to be used in production environments, neither in networks that can be reached by everybody, nor in other uncontrolled settings._

## Implementation of the Quantum Layer

#### Quantum Layer Links: Peers

A quantum layer link peer is the communication partner on a point-to-point link. One has to generate a new peer in the quantum layer in order to communicate with the peer.

#### Beyond Pseudo Random Number Generator (PRNG) based Emulation

Future versions of the quantum layer may include implementations of emulations of real quantum links.

# Interfaces to the proto-kms

## ETSI-Interfaces

There are basically two ETSI interfaces, i.e.,

-   ETSI QKD GS 14 for retrieval of keys by an external entity from the kms
-   ETSI QKD GS 15 for the configuration of the key forwarding process and peers of the kms

By now both are not supported. Please look instead in the above section _Interface between KMS and SDN-Controller_.

# Protocol Flow Charts in Use

This section docucments the current flow charts implemented.

## GRPC Variant for Configuration of Forwarding

This solely depicts the process for KMS1, but omits the to-be-done process for KMS2.

```mermaid
sequenceDiagram
QKDNC->>KMS1: ETSIGetQuantumInterfaces()
KMS1-->>QKDNC: List QuantumInterfaces
QKDNC->>KMS1: ETSIAddKMSPeer(QuantumInterface, KMS2, ListOptions...)
KMS1->>KMS2: InterComCapabilities(Supported_KeyChopLenghts)
KMS2->>KMS1: InterComCapabilitiesReply(Agreed_KeyChopLenght)
KMS1-->>QKDNC: KMSPeer added
QKDNC->>KMS1: AssignForwarding(PathID, PreviousHop, NextHop)
KMS1-->>QKDNC: AssignForwardingReply
```

## (Intended) gNMI/YANG Variant for Configuration of Forwarding

This solely depicts the process for KMS1, but omits the to-be-done process for KMS2.

```mermaid
sequenceDiagram
QKDNC->>KMS1: ETSIGetQuantumInterfaces()
KMS1-->>QKDNC: List QuantumInterfaces
QKDNC->>KMS1: Yang_AddKMSPeer(QuantumInterface, KMS2, ListOptions...)
KMS1-->>QKDNC: gnmi_OK
QKDNC->>KMS1: gnmi_Subscribe(KMS2_Peer_Status)
KMS1-->>QKDNC: gnmi_Subscribe_OK
KMS1->>KMS2: InterComCapabilities(Supported_KeyChopLenghts)
KMS2-->>KMS1: InterComCapabilitiesReply(Agreed_KeyChopLenght)
KMS1->>QKDNC: gnmi_Subscribe_Update(KMS2_Peer_Status)
QKDNC->>KMS1: Yang_GetPeerInfo(KMS2)
KMS1-->>QKDNC: gnmi_OK
QKDNC->>KMS1: Yang_AssignForwarding(PathID, PreviousHop, NextHop)
KMS1-->>QKDNC: gnmi_OK
```

## Forwarding

The flow with one intermediate KMS (i.e., KMS2). This is omitting the request to forward the payload.

```mermaid
sequenceDiagram
KMS1->>KMS2: KeyForwardingRequest(PathID, KeyId, OTP_encrypted(Payload))
KMS2-->>KMS1: KeyForwardingResponse
KMS2->>KMS3: KeyForwardingRequest(PathID, KeyId, OTP_encrypted(Payload))
KMS3-->>KMS2: KeyForwardingResponse
```
