package kmsHandler

import (
	"fmt"

	"code.fbi.h-da.de/danet/gnmi-target/handler"
	"code.fbi.h-da.de/danet/quant/goKMS/config"
	gnmitargetygot "code.fbi.h-da.de/danet/quant/goKMS/model"
	"github.com/openconfig/gnmi/proto/gnmi"
	"github.com/openconfig/ygot/ygot"
)

type KmsHandler struct {
	handler.DefaultPathHandler
	kmsInfo *config.KMSInfo
}

func NewKmsHandler(kmsInfo *config.KMSInfo) *KmsHandler {
	return &KmsHandler{
		DefaultPathHandler: handler.DefaultPathHandler{
			Name: "kms-handler",
			Paths: map[string]struct{}{
				"/kms": {},
			},
		},
		kmsInfo: kmsInfo,
	}
}

func (yh *KmsHandler) Init(config *handler.Config, publishToSubsFunc func([]*gnmi.Notification) error) error {
	yh.Config = config
	yh.PublishToSubs = publishToSubsFunc

	// Fill out the known fields in the ygot structs
	_, err := yh.updateOrCreateKMS(yh.kmsInfo)
	if err != nil {
		return err
	}

	return nil
}

func (yh *KmsHandler) Update(c ygot.ValidatedGoStruct, jobs []*gnmi.Update) error {
	// static information, therefore a update should not do anything
	return nil
}

func (yh *KmsHandler) updateOrCreateKMS(kmsInfo *config.KMSInfo) ([]*gnmi.Notification, error) {
	yh.Config.Lock()
	defer yh.Config.Unlock()

	copyCurrentConfig, err := ygot.DeepCopy(yh.Config.Data)
	if err != nil {
		return nil, err
	}

	newConfig, ok := copyCurrentConfig.(*gnmitargetygot.Gnmitarget)
	if !ok {
		return nil, fmt.Errorf("Wrong type, exptected: %T, got: %T", (*gnmitargetygot.OpenconfigInterfaces_Interfaces)(nil), copyCurrentConfig)
	}

	confKMS := newConfig.GetOrCreateKms()

	confKMS.KmsId = ygot.String(kmsInfo.ID().String())

	confVersionInfo := confKMS.GetOrCreateVersionInformation()
	confVersionInfo.Firmware = ygot.String(kmsInfo.Version().Firmware())
	confVersionInfo.HwVersion = ygot.String(kmsInfo.Version().HardwareVersion())
	confVersionInfo.SwVersion = ygot.String(kmsInfo.Version().SoftwareVersion())

	//validate struct
	if err := newConfig.Validate(); err != nil {
		return nil, err
	}

	notifications, err := ygot.DiffWithAtomic(yh.Config.Data, newConfig)
	if err != nil {
		return nil, err
	}

	yh.Config.Data = newConfig

	return notifications, nil
}
