package crypto

import "fmt"

type OTP struct {
	name string
}

func NewOTP(name string) *OTP {
	return &OTP{
		name: name,
	}
}

func (o *OTP) Encrypt(plaintext []byte, key []byte) ([]byte, []byte, error) {
	plainTextLen := len(plaintext)
	keyLen := len(key)
	if plainTextLen != keyLen {
		return nil, nil, fmt.Errorf("For OTP the length of plaintext and key must not differ. Length of plaintext: %d, length of key: %d", plainTextLen, keyLen)
	}

	ciphertext := make([]byte, plainTextLen)

	for i := range ciphertext {
		ciphertext[i] = plaintext[i] ^ key[i]
	}

	return nil, ciphertext, nil
}

func (o *OTP) Decrypt(nonce, ciphertext []byte, key []byte) ([]byte, error) {
	cipherTextLen := len(ciphertext)
	keyLen := len(key)
	if cipherTextLen != keyLen {
		return nil, fmt.Errorf("For OTP the length of ciphertext and key must not differ. Length of ciphertext: %d, length of key: %d", cipherTextLen, keyLen)
	}

	plaintext := make([]byte, cipherTextLen)

	for i := range plaintext {
		plaintext[i] = ciphertext[i] ^ key[i]
	}

	return plaintext, nil
}

func (o *OTP) Name() string {
	return o.name
}
