package event

import (
	"fmt"
	"sync"
)

var eventBusLock = &sync.Mutex{}
var eventBus *EventBus

type EventBus struct {
	subscribers map[Topic]map[chan<- Event]struct{}
}

func NewEventBus() *EventBus {
	if eventBus == nil {
		// lock access to eventBus
		eventBusLock.Lock()
		defer eventBusLock.Unlock()
		// check if eventBus is still nil
		if eventBus == nil {
			fmt.Println("Initializing KMS EventBus.")
			eventBus = &EventBus{
				subscribers: make(map[Topic]map[chan<- Event]struct{}),
			}
		}
	}
	return eventBus
}

func (b *EventBus) Subscribe(topic Topic) (<-chan Event, error) {
	newSubChan := make(chan Event)

	subs, ok := b.subscribers[topic]
	if !ok {
		initialSub := map[chan<- Event]struct{}{
			newSubChan: {},
		}
		b.subscribers[topic] = initialSub
	} else if subs != nil {
		subs[newSubChan] = struct{}{}
	} else {
		return nil, fmt.Errorf("could not subscribe to topic: %d", topic)
	}

	return newSubChan, nil
}

func (b *EventBus) Publish(event Event) error {
	subs, ok := b.subscribers[event.Topic()]
	if !ok {
		return fmt.Errorf("There are no active subscribers for topic: %d", event.Topic())
	}
	for sub := range subs {
		sub <- event
	}
	return nil
}
